#include "testlib.h"
#include "gen_lib.h"
#include <bits/stdc++.h>

using namespace std;

struct Test {
    int n, group_index, q;
    vector<pair<int, int>> edges, queries;

    void print() {
        println(group_index);
        println(n);
        for (auto [v, u] : edges) {
            println(v + 1, u + 1);
        }

        println(q);
        for (auto [t, id] : queries) {
            println(t, id + 1);
        }
    }
};

Test test;
ArgumentParser parser;

namespace tree_generator {
    void generate_bamboo() {
        for (int i = 0; i + 1 < test.n; i++) {
            test.edges.emplace_back(i, i + 1);
        }
    }

    void generate_segtree() {
        for (int i = 1; i < test.n; i++) {
            test.edges.emplace_back(i, (i - 1) / 2);
        }
    }

    void generate_looong() {
        auto n_groups = parser.getInt("groups");
        auto which_group = random::distributeToGroups(test.n, 0, n_groups, rnd);

        vector<vector<int>> groups(n_groups);
        for (int i = 0; i < test.n; i++) {
            groups[which_group[i]].push_back(i);
        }

        int prev = -1;
        for (const auto &group : groups) {
            auto cur_edges = random::tree::randomPruferCode(group.size(), rnd).edges();
            for (const auto &[v, u] : cur_edges) {
                test.edges.emplace_back(group[v], group[u]);
            }

            int root = group[rnd.next(0, int(group.size()) - 1)];
            if (prev != -1) {
                test.edges.emplace_back(prev, root);
            }
            prev = root;
        }
    }

    void init() {
        const string gen_mode = parser.getString("tmode");
        if (gen_mode == "random") {
            test.edges = random::tree::randomPruferCode(test.n, rnd).edges();
        } else if (gen_mode == "looong") {
            generate_looong();
        } else if (gen_mode == "bamboo") {
            generate_bamboo();
        } else if (gen_mode == "segtree") {
            generate_segtree();
        } else {
            global::logAndThrowException("There is no mode for tree generation: " + gen_mode);
        }
    }
} // namespace tree_generator

namespace queries_generator {
    void generate_random() {
        for (int i = 0; i < test.q; i++) {
            int type = rnd.next(1, 2);
            if (test.q == 1) {
                type = 2;
            }

            if (type == 1) {
                test.queries.emplace_back(type, rnd.next(0, test.n - 2));
            } else {
                test.queries.emplace_back(type, rnd.next(0, test.n - 1));
            }
        }
    }

    void generate_small_subset() {
        int subset_size = parser.getInt("subset");
        auto subset = random::distinctIntegers(0, test.n - 1, subset_size, rnd);

        while (int(test.queries.size()) < test.q) {
            int id = subset[rnd.next(0, subset_size - 1)];
            test.queries.emplace_back(1, id);
            if (rnd.next(0, 1)) {
                test.queries.emplace_back(2, test.edges[id].first);
            } else {
                test.queries.emplace_back(2, test.edges[id].second);
            }
        }

        test.queries.resize(test.q);
    }

    void generate_only_erases() {
        vector<int> order(test.n - 1);
        iota(order.begin(), order.end(), 0);
        random::shuffle(order.begin(), order.end(), rnd);

        for (auto id : order) {
            test.queries.emplace_back(1, id);
            if (rnd.next(0, 1)) {
                test.queries.emplace_back(2, test.edges[id].first);
            } else {
                test.queries.emplace_back(2, test.edges[id].second);
            }
        }

        global::assertf(int(test.queries.size()) >= test.q, "q > 2 * (n - 1)");
        test.queries.resize(test.q);
    }

    void generate_bamboo_parity() {
        global::assertf(test.n % 2 == 0, "n must be even");

        while (int(test.queries.size()) < test.q) {
            test.queries.emplace_back(1, 0);
            test.queries.emplace_back(1, test.n - 2);
            test.queries.emplace_back(2, 1);
        }
        test.queries.resize(test.q);
    }

    void generate_bamboo_erases() {
        for (int l = 0, r = test.n - 2; l < r; l++, r--) {
            test.queries.emplace_back(1, l);
            test.queries.emplace_back(2, l + 1);
            test.queries.emplace_back(1, r);
            test.queries.emplace_back(2, r);
        }
        test.queries.resize(test.q);
    }

    void init() {
        const string gen_mode = parser.getString("qmode");
        if (gen_mode == "random") {
            generate_random();
        } else if (gen_mode == "small_subset") {
            generate_small_subset();
        } else if (gen_mode == "only_erases") {
            generate_only_erases();
        } else if (gen_mode == "bamboo_parity") {
            generate_bamboo_parity();
        } else if (gen_mode == "bamboo_erases") {
            generate_bamboo_erases();
        } else {
            global::logAndThrowException("There is no mode for queries generation: " + gen_mode);
        }
    }
} // namespace queries_generator

int main(int argc, char *argv[]) {
    registerGen(argc, argv, 1);
    parser = ArgumentParser(argc, argv);

    test.n = parser.getInt("n");
    test.group_index = parser.getInt("group");
    test.q = parser.getInt("q");

    tree_generator::init();
    queries_generator::init();

    test.print();
}
