#include "testlib.h"
#include <bits/stdc++.h>

using namespace std;

constexpr int GROUPS = 7;

constexpr int MAX_N[GROUPS + 1]{
    /* 0: */ 5'000'00,
    /* 1: */ 100,
    /* 2: */ 1'000'00,
    /* 3: */ 1'000'00,
    /* 4: */ 1'000'00,
    /* 5: */ 1'000'00,
    /* 6: */ 1'000'00,
    /* 7: */ 3'000'00,
};

struct disjoin_set_union {
    vector<int> par;

    disjoin_set_union(int n) : par(n) {
        iota(par.begin(), par.end(), 0);
    }

    int root(int v) {
        return par[v] == v ? v : (par[v] = root(par[v]));
    }

    bool unite(int v, int u) {
        if (root(v) == root(u)) return false;
        par[root(v)] = root(u);
        return true;
    }
};

bool validateTree(int n, const vector<pair<int, int>> &edges, int group) {
    disjoin_set_union dsu(n);
    for (auto [v, u] : edges) {
        ensuref(dsu.unite(v, u), "Edges does not form a tree");
    }

    // bamboo check
    if (group == 2) {
        for (int i = 0; i < n - 1; i++) {
            ensuref(edges[i].first == i && edges[i].second == i + 1,
                    "In group %d tree must be a bamboo", group);
        }
    }

    // segtree check
    if (group == 4) {
        for (int i = 0; i < n - 1; i++) {
            ensuref(edges[i].first == i + 1 && edges[i].second == (edges[i].first - 1) / 2,
                    "In group %d tree must be segtree-like", group);
        }
    }
}

int main(int argc, char *argv[]) {
    registerValidation(argc, argv);
    const int group = atoi(validator.group().c_str());

    inf.readInt(group, group, "g");
    inf.readEoln();

    int n = inf.readInt(2, MAX_N[group], "n");
    inf.readEoln();

    vector<pair<int, int>> edges;
    for (int i = 1; i <= n - 1; i++) {
        int v = inf.readInt(1, n, "v_" + to_string(i));
        inf.readSpace();
        int u = inf.readInt(1, n, "u_" + to_string(i));
        inf.readEoln();
        edges.emplace_back(v - 1, u - 1);
    }

    validateTree(n, edges, group);

    int q = inf.readInt(1, MAX_N[group], "q");
    inf.readEoln();

    if (group == 3) {
        ensuref(q == 1, "In group %d q must be equal to 1", group);
    }

    vector<bool> edges_state(n - 1, true);

    for (int qid = 1; qid <= q; qid++) {
        int type = inf.readInt(1, 2, "type_" + to_string(qid));
        inf.readSpace();

        if (type == 1) {
            int eid = inf.readInt(1, n - 1, "eid_" + to_string(qid));
            if (group == 5) {
                ensuref(edges_state[eid], "In group %d edges must only erase", group);
            }
            edges_state[eid] = !edges_state[eid];
        } else {
            inf.readInt(1, n, "vid_" + to_string(qid));
        }

        inf.readEoln();
    }

    inf.readEof();
}
